package com.clcbio.client.thin.api;

import java.util.List;

import com.clcbio.client.thin.model.CLCClientSession;
import com.clcbio.client.thin.model.CommonBeanSerialization;
import com.clcbio.client.thin.model.ThinBinaryData;
import com.clcbio.client.thin.model.admin.IndexServerStatus;
import com.clcbio.client.thin.model.admin.ServerStatus;
import com.clcbio.client.thin.model.admin.ThinPluginDescriptor;
import com.clcbio.client.thin.model.admin.configuration.ServerConfiguration;
import com.clcbio.client.thin.model.admin.configuration.ServerConfigurationModule;
import com.clcbio.client.thin.model.bugsubmission.DiagnosticsCheckType;
import com.clcbio.client.thin.model.bugsubmission.DiagnosticsReport;
import com.clcbio.client.thin.model.bugsubmission.SystemInformationQuery;
import com.clcbio.client.thin.model.exceptions.ServiceException;
import com.clcbio.client.thin.model.log.LogQuery;
import com.clcbio.client.thin.model.log.LogQueryResult;
import com.clcbio.client.thin.model.processes.ClcProcessReflection;
import com.clcbio.client.thin.model.processes.ServerProcessManagerProcesses;
import com.clcbio.client.thin.model.safefile.SafeFileLocation;
import com.clcbio.client.thin.model.safefile.SafeFileLocations;

public interface CLCServerApiAdmin extends SerializableGWTTypes {
    /**
     * Authenticates the user on every attached Model of type
     * AuthenticationPersistenceModel
     * 
     * @param username
     * @param password
     * @return SessionId
     * @throws ServiceException
     */
    public CLCClientSession login(String username, String password) throws ServiceException;

    /**
     * Deletes the Session on the Server associated with given sessionId,
     * preserving the SessionContext
     * 
     * @param sessionId
     * @return true if the associated session was found and deleted, false
     *         otherwise.
     */
    public boolean logout(CLCClientSession clientSession);

    /**
     * Deletes the Session on the Server associated with current request. Use
     * this if session is based upon the standard HttpSession maintained by
     * server.
     * 
     * @return true if the associated session was found and deleted, false
     *         otherwise.
     * @deprecated use logout(String sessionID)
     */
    // public boolean logout();

    /**
     * Like any other call, this keeps the session alive, but also returns the
     * associated client session.
     * 
     * @param sessionID
     * @throws ServiceException
     *             if something goes wrong and especially if the sessionID does
     *             not match the one currently associated with the http request.
     */
    public CLCClientSession heartBeat(String sessionID) throws ServiceException;

    public ServerStatus getServerStatus(String sessionID) throws ServiceException;

    public ServerConfiguration setServerConfiguration(String sessionID, ServerConfiguration serverConfiguration)
            throws ServiceException;

    /**
     * @param sessionID
     * @param configurationModule
     *            TODO
     * @return The configuration of the currently runnning server
     * @throws ServiceException
     *             if something goes wrong and especially if the sessionID does
     *             not match the one currently associated with the http request.
     */
    public ServerConfiguration getServerConfiguration(String sessionID, ServerConfigurationModule configurationModule)
            throws ServiceException;

    public ServerProcessManagerProcesses getProcessManagerProcesses(String sessionID) throws ServiceException;

    public ServerProcessManagerProcesses processManagerMoveProcessForward(String sessionID, ClcProcessReflection queuedCommand)
            throws ServiceException;

    public ServerProcessManagerProcesses processManagerMoveProcessBackward(String sessionID, ClcProcessReflection queuedCommand)
            throws ServiceException;

    public LogQueryResult auditLogQuery(String sessionID, LogQuery logQuery) throws ServiceException;

    public List<ThinPluginDescriptor> getInstalledPlugins(String sessionID) throws ServiceException;

    public List<ThinBinaryData> getPlugins(String sessionID, List<ThinPluginDescriptor> pluginList) throws ServiceException;
    
    /**
     * Remove a user account from the authenticator.
     * 
     * @param userId
     *            the unique UserID of the account.
     */
    public void removeUser(String userId, String sessionID) throws ServiceException;

    /**
     * Add a new user to the authenticator
     * 
     * @param userId
     *            the unique UserID of the account
     * @param password
     *            the password for the account
     */
    public void addUser(String userId, String password, String sessionID) throws ServiceException;

    /**
     * Set or change the password for a specific user account
     * 
     * @param userId
     *            the unique UserID of the account
     * @param password
     *            the new password for the account
     */
    public void setPassword(String userId, String password, String sessionID) throws ServiceException;

    public void addUserToGroup(String userId, String groupId, String sessionID) throws ServiceException;

    public void removeUserFromGroup(String userId, String groupId, String sessionID) throws ServiceException;

    public void removeGroup(String groupId, String sessionID) throws ServiceException;

    public void addGroup(String groupId, String sessionID) throws ServiceException;

    public boolean canEditUsersAndGroups(String sessionID) throws ServiceException;

    public String getAdminGroupID(String sessionID) throws ServiceException;

    public boolean isAdmin(String sessionID) throws ServiceException;

    public void setRootPassword(String currentPassword, String newPassword, String sessionID) throws ServiceException;

    public void invalidateUserHttpSession(String adminSessionID, String userSessionId, String invalidationReason)
            throws ServiceException;

    /**
     * Lists the contents of a specific safe file location
     * 
     * @param safeFilesLocation
     *            the specific location to list contents of or null for current
     *            root locations
     * @return the contents of a specific safe file location
     * @throws ServiceException
     *             or NotAuthorizedException if listing outside safe file area
     *             is referenced
     */
    public SafeFileLocations listSafeFilesIn(String sessionID, SafeFileLocation safeFilesLocation) throws ServiceException;

    /**
     * Generate a ZIP archive with selected system files. Job nodes will be
     * queried if set in query.
     * 
     * @param systemInformationQuery
     *            the selected information to be included in the archive
     * @return a ZIP archive containing the information selected to be queried
     *         in the systemInformationQuery
     */
    public ThinBinaryData generateSystemInformationArchive(String sessionID, SystemInformationQuery systemInformationQuery)
            throws ServiceException;

    /**
     * Generates a Server Diagnostics report with results of various consistency
     * and integrity checks.
     * 
     * @param sessionID
     * @param checkType
     *            the type to generate report for. If null, report will consist
     *            of all types.
     * @return
     * @throws ServiceException
     */
    public List<DiagnosticsReport> generateSystemDiagnosticsReport(String sessionID, DiagnosticsCheckType checkType)
            throws ServiceException;

    public IndexServerStatus getIndexServerStatus(String sessionID) throws ServiceException;
    
    public CommonBeanSerialization javaBeanXMLDecode(String sessionID, String serialisedXmlBean) throws ServiceException;
}
