package com.clcbio.client.thin.api;

import java.util.List;

import com.clcbio.client.thin.model.CLCObjectPath;
import com.clcbio.client.thin.model.CLCObjectReference;
import com.clcbio.client.thin.model.ServerInformation;
import com.clcbio.client.thin.model.clcurl.ClcUrlParameter;
import com.clcbio.client.thin.model.constrain.ClcObjectTypeConstraints;
import com.clcbio.client.thin.model.editor.elementinfo.EditableCLCObject;
import com.clcbio.client.thin.model.exceptions.ServiceException;
import com.clcbio.client.thin.model.importexport.ExportPluginReference;
import com.clcbio.client.thin.model.importexport.ImportPluginReference;
import com.clcbio.client.thin.model.legacy.CLCObjectReferences;
import com.clcbio.client.thin.model.persistence.TransportPermissionObject;
import com.clcbio.client.thin.model.query.ModelQueryResults;
import com.clcbio.client.thin.model.tree.CLCTreeItem;
import com.clcbio.client.thin.model.viewer.ClcObjectThin;

/**
 * @author jschultz
 * 
 */
public interface CLCServerApiData extends SerializableGWTTypes {
    /**
     * Searches all attached Models with given query
     * 
     * @param query
     * @param sessionID
     *            the ID from user login
     * @return Search result
     * @throws ServiceException
     */
    public ModelQueryResults query(String query, String sessionID) throws ServiceException;

    /**
     * Fetch the CLCTreeItem corresponding to the objectReference without
     * actually getting the underlying object of the Model
     * 
     * @param objectReference
     * @param sessionID
     *            the ID from user login
     * @return the children of given tree item
     * @throws ServiceException
     */
    public List<CLCTreeItem> getTreeItem(List<ClcUrlParameter> clcUrls, String sessionID) throws ServiceException;

    /**
     * Fetch the children of given tree item without actually getting the
     * underlying objects of the Model
     * 
     * @param objectReference
     * @param sessionID
     *            the ID from user login
     * @return the children of given tree item
     * @throws ServiceException
     */
    public CLCTreeItem getTreeItems(CLCObjectReference objectReference, ClcObjectTypeConstraints constraints, String sessionID) throws ServiceException;

    /**
     * @param objectReference
     * @param sessionID
     * @return a father object with one path all the way to a specific son
     * @throws ServiceException
     */
    public CLCObjectPath getTreeItemPath(ClcUrlParameter objectReference, String sessionID) throws ServiceException;

    /**
     * @param sessionID
     *            the ID from user login
     * @return References to the currently installed Import Plugins
     * @throws ServiceException
     */
    public ImportPluginReference[] getImportPluginReferences(String sessionID) throws ServiceException;

    /**
     * @param sessionID
     *            the ID from user login
     * @return References to the currently installed Export Plugins
     * @throws ServiceException
     */
    public ExportPluginReference[] getExportPluginReferences(String sessionID) throws ServiceException;

    /**
     * @param clcObjectReferences
     * @param sessionID
     *            the ID from user login
     * @return References to the currently installed Export Plugins that are
     *         able to export the objects referenced by clcObjectReferences
     * @throws ServiceException
     */
    public ExportPluginReference[] getExportPluginReferencesSuitable(List<CLCObjectReference> clcObjectReferences, String sessionID)
            throws ServiceException;

    /**
     * Fetches the thin representation of a ClcObject with values from the
     * actual underlying object in the Model
     * 
     * @param cLCTreeItem
     * @param sessionID
     *            the ID from user login
     * @return the corresponding thin object
     * @throws ServiceException
     */
    public ClcObjectThin getClcObjectThin(CLCObjectReference objectReference, String sessionID) throws ServiceException;

    /**
     * Updates the posted attributes on the referenced CLC Object
     * 
     * @param updatedObject
     * @param sessionID
     *            the ID from user login
     * @return
     * @throws ServiceException
     */
    public EditableCLCObject updateObject(EditableCLCObject updatedObject, String sessionID) throws ServiceException;

    /**
     * Creates a new sub container in the container referenced by
     * persistenceContainerInto, before the container referenced
     * persistenceStructureBefore.
     * 
     * @param containerName
     *            name of new container
     * @param persistenceContainerInto
     *            reference to the container in which the new container will be
     *            made
     * @param persistenceStructureBefore
     *            reference to the container that the new container will reside
     *            next to
     * @return the reference to the newly created container
     * @throws ServiceException
     */
    public CLCObjectReference createContainerBefore(String containerName, CLCObjectReference persistenceContainerInto,
            CLCObjectReference persistenceStructureBefore, String sessionID) throws ServiceException;

    /**
     * Removes the objects from a container The objects will be moved to the
     * Recycle Bin
     * 
     * @param persistenceStructures
     *            references to the objects that should be moved to the Recycle
     *            Bin
     * @throws ServiceException
     */
    public void removeContainers(CLCObjectReferences persistenceStructures, String sessionID) throws ServiceException;

    /**
     * Empties the logical RecycleBin of the referenced Model
     * 
     * @param persistenceModelRef
     * @throws ServiceException
     */
    public void emptyRecycleBin(String persistenceModelId, String sessionId) throws ServiceException;
    /**
     * List of permissions for a given persistence container.
     * 
     * @param pmId
     * @param containerId
     * @param sessionId
     * @return
     * @throws ServiceException
     */
    public List<TransportPermissionObject> getPermissions(String pmId, String containerId, String sessionId) throws ServiceException;

    /**
     * Set a permission for a given container
     * 
     * @param pmId
     * @param containerId
     * @param sessionId
     * @param groupId
     * @param canRead
     * @param canWrite
     * @param recurse
     * @throws ServiceException
     */
    public void setPermissions(String pmId, String containerId, String groupId, boolean canRead, boolean canWrite, boolean recurse,
            String sessionId) throws ServiceException;

    public List<String> getGroups(String userId, String sessionId) throws ServiceException;

    public List<String> getAllGroups(String sessionId) throws ServiceException;

    public List<String> getUsersInGroup(String groupId, String sessionId) throws ServiceException;

    public List<String> getUsers(String session) throws ServiceException;

    public ServerInformation getServerInformation() throws ServiceException;

}
