package com.clcbio.client.thin.api;

import java.util.List;

import com.clcbio.client.thin.model.ClcUrlParsed;
import com.clcbio.client.thin.model.CloneTransport;
import com.clcbio.client.thin.model.admin.configuration.subsystem.templatecommand.InstalledTemplateCommand;
import com.clcbio.client.thin.model.admin.configuration.subsystem.workflow.InstalledWorkflowPlugin;
import com.clcbio.client.thin.model.admin.workflow.InstalledWorkflows;
import com.clcbio.client.thin.model.clcurl.ClcUrlParameter;
import com.clcbio.client.thin.model.command.ServerCommandOutput;
import com.clcbio.client.thin.model.command.ServerCommandReference;
import com.clcbio.client.thin.model.exceptions.ServiceException;
import com.clcbio.client.thin.model.executionenvironment.ServerExecutionEnvironment;
import com.clcbio.client.thin.model.processes.ClcProcessReflection;
import com.clcbio.client.thin.model.processes.UserProcesses;
import com.clcbio.client.thin.model.qualification.QualificationReport;
import com.clcbio.client.thin.model.qualification.QualificationReportWorkflow;
import com.clcbio.client.thin.model.schedule.Interval;
import com.clcbio.client.thin.model.schedule.ScheduledCommand;

public interface CLCServerApiProcess extends SerializableGWTTypes {
    
    /**
     * Get a list of references to the Commands currently loaded on the server,
     * omitting those only registered
     * 
     * @param sessionID
     *            ID from user login
     * @return array of references corresponding to the Commands currently
     *         loaded by the CLC Server
     */
    public ServerCommandReference[] getServerCommandRefsLoaded(String sessionID) throws ServiceException;
    
    /**
     * Get reference to the Command currently loaded on the server,
     * omitting those only registered
     * 
     * @param sessionID
     *            ID from user login
     * @param serverCommandId id of the server command
     * @return The reference corresponding to the Command with the id, currently
     *         loaded by the CLC Server
     */
    public ServerCommandReference getServerCommandRefLoaded(String sessionID, String serverCommandId) throws ServiceException;

    /**
     * Get a list of references to the Registered Commands available on the
     * server. A registered command need not to be loaded, while a loaded is
     * always registered.
     * 
     * @param sessionID
     *            ID from user login
     * @return array of references corresponding to the Commands currently
     *         loaded by the CLC Server
     */
    public ServerCommandReference[] getServerCommandRefsRegistered(String sessionID) throws ServiceException;
    
    /**
     * Execution or queuing of a server Command
     * 
     * @param sessionID
     *            ID from user login
     * @param commandReference
     * @return the Id of the process initiated or queued on the server
     * @throws ServiceException
     */
    public String queueExecute(String sessionID, ServerCommandReference commandReference) throws ServiceException;
    
    /**
     * Schedulation of a server Command
     * 
     * @param sessionID
     *            ID from user login
     * @param commandReference
     * @return the Id of the process scheduled on the server
     * @throws ServiceException
     */
    public String schedule(String sessionID, ServerCommandReference commandReference, List<Interval> intervals)
            throws ServiceException;
    
    /**
     * @return the status of processes started by this session
     */
    public UserProcesses getSessionProcesses(String sessionID) throws ServiceException;
    
    /**
     * @return the status of the specific processes started by this session
     */
    public ClcProcessReflection getSessionProcess(String sessionID, String processID) throws ServiceException;
    
    /**
     * @param sessionID
     *            ID from user login
     * @param processID
     *            id of server process to stop
     * @throws ServiceException
     */
    public void doStopProcess(String sessionID, String processID) throws ServiceException;
    
    /**
     * @param sessionID
     *            ID from user login
     * @param processID
     *            id of server process to stop and requeue
     * @throws ServiceException
     */
    public void doStopRequeueProcess(String sessionID, String processID) throws ServiceException;
    
    /**
     * @param sessionID
     *            ID from user login
     * @param processID
     *            id of server process to suspend
     * @throws ServiceException
     */
    public void doSuspendProcess(String sessionID, String processID) throws ServiceException;
    
    /**
     * @param sessionID
     *            ID from user login
     * @param processID
     *            id of server process to resume
     * @throws ServiceException
     */
    public void doResumeProcess(String sessionID, String processID) throws ServiceException;
    
    /**
     * @param sessionID
     * @param processID
     * @throws ServiceException
     */
    public void doAbort(String sessionID, String processID) throws ServiceException;
    
    /**
     * Removes terminated process output. If this is a master process, any sub process output will be removed also.
     * @param sessionID
     * @param processID
     * @throws ServiceException if process was not found or still running
     */
    public void doRemoveTerminatedProcess(String sessionID, String processID) throws ServiceException;
    
    /**
     * @param sessionID
     * @param processID
     * @return an empty output object from the process, representing the output
     *         type for this command
     * @throws ServiceException
     */
    public ServerCommandOutput getProcessOutputType(String sessionID, String processID) throws ServiceException;
    
    /**
     * @param sessionID
     * @param processID
     * @return The output from the process
     * @throws ServiceException
     */
    public ServerCommandOutput getProcessOutput(String sessionID, String processID) throws ServiceException;
    
    /**
     * Returns list of supported Execution Environments determined from current
     * server configuration. If no execution environments are returned, any
     * normal Command Queued at this server will stay in queue until an
     * execution environments is dynamically added to the running server, or the
     * job is removed again.
     * 
     * @param sessionID
     * @return list of supported Execution Environments determined as from
     *         current server configuration
     * @throws ServiceException
     */
    public List<ServerExecutionEnvironment> getExecutionEnvironments(String sessionID) throws ServiceException;
    
    public void deleteScheduledCommand(String sessionID, String scheduleId) throws ServiceException;
    
    public List<ScheduledCommand> getScheduledCommands(String sessionID) throws ServiceException;
    
    public void updateScheduledCommand(String sessionID, ScheduledCommand scheduledCommand) throws ServiceException;
    
    public ClcUrlParsed parseClcUrl(String sessionID, ClcUrlParameter clcUrlParameter) throws ServiceException;
    
    public InstalledWorkflows workflowManagerList(String sessionID) throws ServiceException;
    
    public QualificationReportWorkflow workflowManagerInstall(String sessionID, InstalledWorkflowPlugin newWorkflow) throws ServiceException;
    
    public void workflowManagerUnInstall(String sessionID, String installedWorkflowPluginId) throws ServiceException;
    
    public QualificationReportWorkflow workflowManagerUpdate(String sessionID, String pluginToUpdateId, InstalledTemplateCommand updatedCommand) throws ServiceException;
        
    /**
     * Analyzes the executionCandidate and Produces a report that details the execution readiness of the supplied values.
     * This can be used as a pre-execution step to assert the likely success of the upcoming execution.
     * @param sessionID
     * @param executionCandidate the parameters to be checked. The same as would be given to a queueExecute() invocation;
     * @return The qualification report for the executionCandidate
     * @throws ServiceException
     */
    public QualificationReport qualify(String sessionID, ServerCommandReference executionCandidate) throws ServiceException;

    /**
     * The non efficient but secure way, serialization mechanism wise, to leave the
     * Object Graph cloning to one of the nursed, accepted serialization mechanisms,
     * rather than leaving this to a local new third party library, which might
     * lower the bar even further than the current lowest common denominator of
     * GWT/JSON/JAX_B/JAX_WS/JSR57 which is hard enough to satisfy as it is. YES,
     * this means an unnecessary round trip to the server. As said, it is non
     * efficient, but think of the future. NO, GWT serialization is asymmetric 
     * so you can't do it solely client side in your GWT app. 
     * 
     * @param sessionID
     * @param cloneTransport object graph already cloned for you by the serialization mechanism
     * @return clone
     * @throws ServiceException
     */
    public CloneTransport remoteClone(String sessionID, CloneTransport cloneTransport) throws ServiceException;
}
