package com.clcbio.client.thin.interpretation;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.clcbio.client.thin.model.admin.configuration.ServerConfigurationModule;
import com.clcbio.client.thin.model.admin.configuration.subsystem.permission.GrantedGroups;
import com.clcbio.client.thin.model.admin.configuration.subsystem.permission.GrantedGroupsKey;
import com.clcbio.client.thin.model.admin.configuration.subsystem.permission.KeyType;
import com.clcbio.client.thin.model.admin.configuration.subsystem.permission.ServerApiPermissionConfiguration;
import com.clcbio.client.thin.model.command.ServerCommandReference;

/**
 * @author jdmeza The thin interpretation layer upon the raw configuration file
 */
public class GrantConfigurationInterpreter {
    
    /**
     * Retrieves the GrantedGroups configuration object from the
     * grantsConfigration associated with given configurationModule and
     * moduleGrantedGroupsId. If no GrantedGroups is associated in given
     * grantsConfigration null is returned.
     * 
     * @param grantsConfigration
     * @param configurationModule
     * @param moduleGrantedGroupsId
     * @return GrantedGroups configuration object or null
     */
    public static GrantedGroups getGrantedGroupsFor(ServerApiPermissionConfiguration grantsConfigration,
            ServerConfigurationModule
            configurationModule, String moduleGrantedGroupsId) {
        Map<GrantedGroupsKey, GrantedGroups> map = grantsConfigration.getPermissionsMap();
        GrantedGroupsKey key = createFrom(configurationModule, moduleGrantedGroupsId);
        return map.get(key);
    }
    
    /**
     * Retrieves the GrantedGroups configuration object from the
     * grantsConfigration associated with given commandReference. If no
     * GrantedGroups is associated in given grantsConfigration null is returned.
     * 
     * @param grantsConfigration
     * @param commandReference
     * @return GrantedGroups configuration object or null
     */
    public static GrantedGroups getGrantedGroupsFor(ServerApiPermissionConfiguration grantsConfigration, ServerCommandReference
            commandReference) {
        Map<GrantedGroupsKey, GrantedGroups> map = grantsConfigration.getPermissionsMap();
        GrantedGroupsKey key = createFrom(commandReference);
        return map.get(key);
    }
    
    /**
     * The encapsulation of the ServerConfigurationModule to GrantedGroupsKey
     * convention
     * 
     * @param configurationModule
     * @param moduleGrantedGroupsId
     * @return
     */
    public static GrantedGroupsKey createFrom(ServerConfigurationModule configurationModule, String moduleGrantedGroupsId) {
        GrantedGroupsKey key = new GrantedGroupsKey();
        key.setKey(configurationModule.name());
        key.setKeyType(KeyType.PERMISION_CONFIGURATION);
        key.setSubType(moduleGrantedGroupsId);
        key.setDescription(configurationModule.getConfigurationTypeDescription());
        return key;
    }
    
    /**
     * The encapsulation of the ServerCommandReference to GrantedGroupsKey
     * convention
     * 
     * @param commandReference
     * @return
     */
    public static GrantedGroupsKey createFrom(ServerCommandReference commandReference) {
        GrantedGroupsKey key = new GrantedGroupsKey();
        key.setKey(commandReference.getId());
        key.setKeyType(KeyType.PERMISSION_COMMAND);
        key.setDescription(commandReference.getName());
        return key;
    }
    
    /**
     * Return a list of group names reflecting the intersection of the granted
     * group names for the GrantedGroups and any enabled child GrantedGroups
     * recursively. 
     * 
     * This means, that if the enabled attribute of a GrantedGroups object is set to false
     * this objects list of group names is not part of the intersection selection.
     * If all GrantedGroups objects are disabled, null will be returned.
     * 
     * @param grantsConfigration
     * @param grantedGroups
     * @return List of granted group names or null if grants are disabled
     */
    public static Set<String> getGrantedGroupNamesIntersectionFor(ServerApiPermissionConfiguration grantsConfigration, GrantedGroups grantedGroups) {
        Map<GrantedGroupsKey, GrantedGroups> map = grantsConfigration.getPermissionsMap();
        Set<String> result = getNamesFor(grantedGroups, map);
        return result;
    }
    
    private static Set<String> getNamesFor(GrantedGroups commandGrants, Map<GrantedGroupsKey, GrantedGroups> map) {
        Set<String> thisSet = null;
        GrantedGroups childGrantedGroups = null;
        if(commandGrants != null){
            if(commandGrants.isPermissionsEnabled()){
                thisSet = commandGrants.getNameOfGrantedGroups();
            }
            GrantedGroupsKey cp = commandGrants.getChildPermission();
            if(cp != null){
               childGrantedGroups = map.get(cp);
            }
            return intersectionOf(thisSet, getNamesFor(childGrantedGroups, map));
        }
        else{
            return null;
        }
    }
    private static Set<String> intersectionOf(Set<String> a, Set<String> b) {
        HashSet<String> result = new HashSet<String>();
        if(a == null && b == null){
            return null;
        }
        if(a == null){
            result.addAll(b);
        }
        else if(b == null){
            result.addAll(a);
        }
        else{
            for (String nameOfGrantedGroup : a) {
                if(b.contains(nameOfGrantedGroup)){
                    result.add(nameOfGrantedGroup);
                }
            }
        }
        return result;
    }
    
}
